'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var tslib = require('tslib');
var core = require('@apollo/client/core');
var utilities = require('@apollo/client/utilities');

var getDirectiveArgumentValueFromOperation = function (operation, directiveName, argumentName) {
    var _a, _b, _c, _d, _e, _f;
    return (_f = (_e = (_d = (_c = (_b = (_a = operation.query.definitions.find(function (definition) { return definition.kind === 'OperationDefinition'; })) === null || _a === void 0 ? void 0 : _a.directives) === null || _b === void 0 ? void 0 : _b.find(function (directive) { var _a; return ((_a = directive.name) === null || _a === void 0 ? void 0 : _a.value) === directiveName; })) === null || _c === void 0 ? void 0 : _c.arguments) === null || _d === void 0 ? void 0 : _d.find(function (argument) { var _a; return ((_a = argument.name) === null || _a === void 0 ? void 0 : _a.value) === argumentName; })) === null || _e === void 0 ? void 0 : _e.value) === null || _f === void 0 ? void 0 : _f.value;
};
var prefixTypenames = function (data, apiName) {
    if (data == null || typeof data !== 'object' || apiName === 'default') {
        return data;
    }
    if (Array.isArray(data)) {
        return data.map(function (item) { return prefixTypenames(item, apiName); });
    }
    var newData = Object.entries(data).reduce(function (ctx, _a) {
        var _b;
        var itemKey = _a[0], item = _a[1];
        return (tslib.__assign(tslib.__assign({}, ctx), (_b = {}, _b[itemKey] = prefixTypenames(item, apiName), _b)));
    }, {});
    if (data.__typename) {
        return tslib.__assign(tslib.__assign({}, newData), { __typename: "".concat(apiName, ":").concat(data.__typename) });
    }
    return newData;
};
var isFunction = function (fn) {
    return fn && {}.toString.call(fn) === '[object Function]';
};

var MultiAPILink = /** @class */ (function (_super) {
    tslib.__extends(MultiAPILink, _super);
    function MultiAPILink(config, request) {
        var _this = _super.call(this, request) || this;
        _this.config = config;
        _this.httpLink = config.createHttpLink();
        _this.wsLinks = {};
        return _this;
    }
    MultiAPILink.prototype.request = function (operation, forward) {
        var _a, _b, _c;
        if ((!utilities.hasDirectives(['api'], operation.query) &&
            !this.config.defaultEndpoint) ||
            utilities.hasDirectives(['rest'], operation.query)) {
            return (_a = forward === null || forward === void 0 ? void 0 : forward(operation)) !== null && _a !== void 0 ? _a : null;
        }
        var apiName = getDirectiveArgumentValueFromOperation(operation, 'api', 'name');
        if (!apiName) {
            var contextKey = getDirectiveArgumentValueFromOperation(operation, 'api', 'contextKey');
            if (contextKey) {
                apiName = operation.getContext()[contextKey];
            }
            else if (!!this.config.defaultEndpoint) {
                apiName = this.config.defaultEndpoint;
            }
        }
        var query = utilities.removeDirectivesFromDocument([{ name: 'api', remove: true }], operation.query);
        if (!query) {
            throw new Error('Error while removing directive api');
        }
        operation.query = query;
        if (this.config.getContext) {
            operation.setContext(this.config.getContext(apiName, operation.getContext));
        }
        if (this.config.endpoints[apiName]) {
            operation.setContext({
                uri: "".concat(this.config.endpoints[apiName]).concat((_b = this.config.httpSuffix) !== null && _b !== void 0 ? _b : '/graphql'),
            });
        }
        else if (process.env.NODE_ENV === 'dev') {
            throw new Error("".concat(apiName, " is not defined in endpoints definitions"));
        }
        var definition = utilities.getMainDefinition(operation.query);
        if (definition.kind === 'OperationDefinition' &&
            definition.operation === 'subscription') {
            if (typeof window === 'undefined') {
                return null;
            }
            if (!this.config.createWsLink) {
                throw new Error("You tried to call a subscription without configuring \"createWsLink\" function:".concat(operation.query));
            }
            if (!this.wsLinks[apiName]) {
                var endpoint = this.config.endpoints[apiName];
                var wsEndpoint = endpoint.startsWith('/')
                    ? "".concat(window.location.origin).concat(endpoint).replace('http', 'ws')
                    : endpoint.replace('http', 'ws');
                this.wsLinks[apiName] = this.config.createWsLink("".concat(wsEndpoint).concat((_c = this.config.wsSuffix) !== null && _c !== void 0 ? _c : '/graphql/subscriptions'));
            }
            var response_1 = this.wsLinks[apiName].request(operation, forward);
            if (this.config.prefixTypenames && isFunction(response_1 === null || response_1 === void 0 ? void 0 : response_1.map)) {
                return response_1.map(function (e) { return prefixTypenames(e, apiName); });
            }
            return response_1;
        }
        var response = this.httpLink.request(operation, forward);
        if (this.config.prefixTypenames && isFunction(response === null || response === void 0 ? void 0 : response.map)) {
            return response.map(function (e) { return prefixTypenames(e, apiName); });
        }
        return response;
    };
    return MultiAPILink;
}(core.ApolloLink));

exports.MultiAPILink = MultiAPILink;
